/***
 * Excerpted from "No Fluff Just Stuff 2007",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs07 for more book information.
***/
package org.apache.tapestry.integration;

import org.openqa.selenium.server.SeleniumServer;
import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.thoughtworks.selenium.DefaultSelenium;
import com.thoughtworks.selenium.Selenium;


@Test(timeOut = 50000, sequential = true, groups =
{ "integration" })
public class IntegrationTests extends Assert
{
    private static final int JETTY_PORT = 9999;

    private static final String BASE_URL = 
	     String.format("http://localhost:%d/", JETTY_PORT);

    /** 60 seconds */
    public static final String PAGE_LOAD_TIMEOUT = "600000";

    private Selenium _selenium;

    private SeleniumServer _server;

    private JettyRunner _jettyRunner;
    
	
	
	
	
    @BeforeClass
    public void startup() throws Exception
    {
        _jettyRunner = new JettyRunner("/", JETTY_PORT, "src/test/app1");

        _server = new SeleniumServer();

        _server.start();

        _selenium = new DefaultSelenium("localhost", SeleniumServer.DEFAULT_PORT, 
                "*firefox", BASE_URL);

        _selenium.start();
    }

    @AfterClass
    public void cleanup() throws Exception
    {
        _selenium.stop();

        _selenium = null;

        _server.stop();
        _server = null;

        _jettyRunner.stop();
        _jettyRunner = null;
    }

    @Test
    public void app1_basic_output() throws Exception
    {
        _selenium.open(BASE_URL);

        clickAndWait("link=Start Page");

        // This comes from the Border cmponent's template

        assertTrue(_selenium.getTitle().contains("Tapestry"));

        // Text from Start.html

        assertTextPresent("First Tapestry 5 Page");

        // This is text passed from Start.html to Output as a parameter

        assertTextPresent("we have basic parameters working");
    }



    private void clickAndWait(String link)
    {
        _selenium.click(link);
        _selenium.waitForPageToLoad(PAGE_LOAD_TIMEOUT);
    }

    private void assertTextPresent(String text)
    {
        if (_selenium.isTextPresent(text))
            return;

        System.err.printf("Text pattern '%s' not found in:\n%s\n\n", 
                text, _selenium.getHtmlSource());

        throw new AssertionError("Page did not contain '" + text + "'.");
    }
}
