/***
 * Excerpted from "No Fluff Just Stuff 2007",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs07 for more book information.
***/
package com.howardlewisship.easymockexamples;

import static org.easymock.EasyMock.expect;

import org.easymock.EasyMock;
import org.easymock.IMocksControl;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

public class OrderProcessorImplTest extends Assert
{
    public static final String PRODUCT_ID = "prod-0001";

    private IMocksControl _control;

    @BeforeMethod
    public final void setup()
    {
        _control = EasyMock.createControl();
    }

    protected final <T> T newMock(Class<T> mockClass)
    {
        return _control.createMock(mockClass);
    }

    protected final void replay()
    {
        _control.replay();
    }

    protected final void verify()
    {
        _control.verify();
    }

    @Test
    public void sufficient_quantity_available()
    {
        InventoryManager manager = newMock(InventoryManager.class);
        Order order = newMock(Order.class);
        
        expect(manager.quantityAvailable(PRODUCT_ID)).andReturn(100);

        order.addLineItem(PRODUCT_ID, 50, false);

        replay();

        OrderProcessor processor = new OrderProcessorImpl(manager);

        processor.processLineItem(PRODUCT_ID, 50, order);

        verify();
    }
}
